/*
 * JPolyliner2View.java
 */

package jpolyliner2;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import jpolyliner2.utils.CreateRemoveAction;
import jpolyliner2.utils.InsertPolyPointsOnClickAction;
import jpolyliner2.utils.NoOpAction;
import jpolyliner2.utils.RemovePolyPointsOnClickAction;
import jpolyliner2.utils.StrategyAction;
import polyliner.domain.impl.facade.scalesnap.ScaleSnapGrid;
import polyliner.domain.impl.polyline.simple.DefaultPolyline;
import polyliner.domain.interfaces.Polyline;
import polyliner.domain.utilities.draw.DrawPointManager;
import polyliner.domain.utilities.draw.DrawPolylineManager;
import polyliner.domain.utilities.draw.impl.DefaultDrawPointFactory;
import polyliner.domain.utilities.draw.impl.DefaultDrawPolylineFactory;
import polyliner.exporters.Exporter;
import polyliner.exporters.impl.DefaultExporter;
import polyliner.exporters.impl.PlasmExporter;
import polyliner.importers.Importer;
import polyliner.importers.impl.DefaultScaleSnapImporter;

/**
 * The application's main frame.
 */
public class JPolyliner2View extends FrameView implements MouseListener, MouseMotionListener {

    // Polylines created
    private List<Polyline> polylines = new LinkedList<Polyline>();
    // Grid with scale and snap
    private ScaleSnapGrid grid;
    // Manager for point drawing
    private DrawPointManager pointManager;
    // Manager for polyline drawing
    private DrawPolylineManager polylineManager;
    
    // Panel on which draw the image
    private ImageDrawComponent imageComponent;
    // Image loaded
    private BufferedImage currentImage = null;
    
    // State/Strategy pattern for actions
    private StrategyAction currentMode;
    // Current Polyline
    private Polyline currentPolyline = null;
    // Id counter for default names
    private int idcounter = 0;
    
    // File Chooser
    private JFileChooser saveDialog = new JFileChooser();
    private JFileChooser loadProjectDialog = new JFileChooser();
    private JFileChooser loadImageDialog = new JFileChooser();
    private JFileChooser exportDialog = new JFileChooser();
    
    public JPolyliner2View(SingleFrameApplication app) {
        super(app);

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = JPolyliner2App.getApplication().getMainFrame();
            aboutBox = new JPolyliner2AboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        JPolyliner2App.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        toolPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        scaleBox = new javax.swing.JComboBox();
        gridBox = new javax.swing.JComboBox();
        snapToggleButton = new javax.swing.JToggleButton();
        jLabel4 = new javax.swing.JLabel();
        createPointsButton = new javax.swing.JToggleButton();
        insertPolyPointsButton = new javax.swing.JToggleButton();
        jLabel5 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        createPolyByClickButton = new javax.swing.JToggleButton();
        removeSelPolylineButton = new javax.swing.JButton();
        createPolylineButton = new javax.swing.JButton();
        noActionButton = new javax.swing.JToggleButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        imageContainerPanel = new javax.swing.JPanel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        loadImageMenuItem = new javax.swing.JMenuItem();
        loadProjectMenu = new javax.swing.JMenuItem();
        saveProjectMenu = new javax.swing.JMenuItem();
        exportProjectMenu = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.BorderLayout());

        toolPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        toolPanel.setName("toolPanel"); // NOI18N
        toolPanel.setPreferredSize(new java.awt.Dimension(280, 100));

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(jpolyliner2.JPolyliner2App.class).getContext().getResourceMap(JPolyliner2View.class);
        jLabel1.setFont(resourceMap.getFont("jLabel1.font")); // NOI18N
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        scaleBox.setName("scaleBox"); // NOI18N
        scaleBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scaleBoxActionPerformed(evt);
            }
        });

        gridBox.setName("gridBox"); // NOI18N
        gridBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gridBoxActionPerformed(evt);
            }
        });

        snapToggleButton.setText(resourceMap.getString("snapToggleButton.text")); // NOI18N
        snapToggleButton.setName("snapToggleButton"); // NOI18N
        snapToggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                snapToggleButtonActionPerformed(evt);
            }
        });

        jLabel4.setFont(resourceMap.getFont("jLabel4.font")); // NOI18N
        jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        createPointsButton.setText(resourceMap.getString("createPointsButton.text")); // NOI18N
        createPointsButton.setName("createPointsButton"); // NOI18N
        createPointsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createPointsButtonActionPerformed(evt);
            }
        });

        insertPolyPointsButton.setText(resourceMap.getString("insertPolyPointsButton.text")); // NOI18N
        insertPolyPointsButton.setName("insertPolyPointsButton"); // NOI18N
        insertPolyPointsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                insertPolyPointsButtonActionPerformed(evt);
            }
        });

        jLabel5.setFont(resourceMap.getFont("jLabel5.font")); // NOI18N
        jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jList1.setName("jList1"); // NOI18N
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList1MouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        createPolyByClickButton.setText(resourceMap.getString("createPolyByClickButton.text")); // NOI18N
        createPolyByClickButton.setName("createPolyByClickButton"); // NOI18N
        createPolyByClickButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createPolyByClickButtonActionPerformed(evt);
            }
        });

        removeSelPolylineButton.setText(resourceMap.getString("removeSelPolylineButton.text")); // NOI18N
        removeSelPolylineButton.setName("removeSelPolylineButton"); // NOI18N
        removeSelPolylineButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeSelPolylineButtonActionPerformed(evt);
            }
        });

        createPolylineButton.setText(resourceMap.getString("createPolylineButton.text")); // NOI18N
        createPolylineButton.setName("createPolylineButton"); // NOI18N
        createPolylineButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createPolylineButtonActionPerformed(evt);
            }
        });

        noActionButton.setText(resourceMap.getString("noActionButton.text")); // NOI18N
        noActionButton.setName("noActionButton"); // NOI18N
        noActionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noActionButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout toolPanelLayout = new javax.swing.GroupLayout(toolPanel);
        toolPanel.setLayout(toolPanelLayout);
        toolPanelLayout.setHorizontalGroup(
            toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(noActionButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(createPointsButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, toolPanelLayout.createSequentialGroup()
                        .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(snapToggleButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                            .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, toolPanelLayout.createSequentialGroup()
                                .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 76, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(gridBox, 0, 164, Short.MAX_VALUE)
                                    .addComponent(scaleBox, 0, 164, Short.MAX_VALUE))))
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, toolPanelLayout.createSequentialGroup()
                        .addComponent(removeSelPolylineButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(createPolylineButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(jLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, toolPanelLayout.createSequentialGroup()
                        .addComponent(createPolyByClickButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(insertPolyPointsButton, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(toolPanelLayout.createSequentialGroup()
                        .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                        .addGap(5, 5, 5))))
        );
        toolPanelLayout.setVerticalGroup(
            toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(scaleBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(toolPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(gridBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(snapToggleButton)
                .addGap(18, 18, 18)
                .addComponent(jLabel4)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(noActionButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(createPointsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(createPolyByClickButton, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(insertPolyPointsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel5)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 226, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(createPolylineButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(removeSelPolylineButton)
                .addGap(38, 38, 38))
        );

        mainPanel.add(toolPanel, java.awt.BorderLayout.LINE_END);

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        imageContainerPanel.setName("imageContainerPanel"); // NOI18N
        imageContainerPanel.setLayout(new java.awt.BorderLayout());
        jScrollPane2.setViewportView(imageContainerPanel);

        mainPanel.add(jScrollPane2, java.awt.BorderLayout.CENTER);

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        loadImageMenuItem.setText(resourceMap.getString("loadImageMenuItem.text")); // NOI18N
        loadImageMenuItem.setName("loadImageMenuItem"); // NOI18N
        loadImageMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadImageMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(loadImageMenuItem);

        loadProjectMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        loadProjectMenu.setText(resourceMap.getString("loadProjectMenu.text")); // NOI18N
        loadProjectMenu.setName("loadProjectMenu"); // NOI18N
        loadProjectMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadProjectMenuActionPerformed(evt);
            }
        });
        fileMenu.add(loadProjectMenu);

        saveProjectMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        saveProjectMenu.setText(resourceMap.getString("saveProjectMenu.text")); // NOI18N
        saveProjectMenu.setName("saveProjectMenu"); // NOI18N
        saveProjectMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveProjectMenuActionPerformed(evt);
            }
        });
        fileMenu.add(saveProjectMenu);

        exportProjectMenu.setText(resourceMap.getString("exportProjectMenu.text")); // NOI18N
        exportProjectMenu.setName("exportProjectMenu"); // NOI18N
        exportProjectMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exportProjectMenuActionPerformed(evt);
            }
        });
        fileMenu.add(exportProjectMenu);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(jpolyliner2.JPolyliner2App.class).getContext().getActionMap(JPolyliner2View.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 483, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

  
    
    private void loadImageMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadImageMenuItemActionPerformed
 
        this.loadImageDialog.setMultiSelectionEnabled(false);
        int res = this.loadImageDialog.showOpenDialog(this.mainPanel);
        File f = loadImageDialog.getSelectedFile();
        if(res == JFileChooser.APPROVE_OPTION && f != null) {
            try {
                clearGridData();
                if(this.currentImage != null) {
                    this.currentImage.flush();
                    this.currentImage = null;
                }

                this.currentImage = ImageIO.read(f);
            } catch (IOException e) {
                /* Warn the user about file */
                JOptionPane.showMessageDialog(this.mainPanel,"Exception while loading file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
            }  

            this.grid = new ScaleSnapGrid(currentImage.getWidth(),currentImage.getHeight(),1,false);
            this.pointManager = new DrawPointManager(grid, new DefaultDrawPointFactory(1, 15));
            this.polylineManager = new DrawPolylineManager(new DefaultDrawPolylineFactory());

            this.currentMode = new NoOpAction();
            setToggleButtons(this.noActionButton);
            endLoading();
            updatePolyList();
        }
       
    }//GEN-LAST:event_loadImageMenuItemActionPerformed

    private void noActionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noActionButtonActionPerformed
        if(this.currentImage != null) {
            this.currentMode.clear();
            this.currentMode = new NoOpAction();
            setToggleButtons(evt.getSource());
        } else {
            setToggleButtons("");
        }
    }//GEN-LAST:event_noActionButtonActionPerformed

    private void createPointsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createPointsButtonActionPerformed
        if(this.currentImage != null) {
            this.currentMode.clear();
            this.currentMode = new CreateRemoveAction(grid, pointManager, polylineManager);
            setToggleButtons(evt.getSource());
        } else {
            setToggleButtons("");
        }
    }//GEN-LAST:event_createPointsButtonActionPerformed

    private void insertPolyPointsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_insertPolyPointsButtonActionPerformed
        if(this.currentImage != null) {
            this.currentMode.clear();
            this.currentMode = new RemovePolyPointsOnClickAction(grid, pointManager, polylineManager);
            setToggleButtons(evt.getSource());
        } else {
            setToggleButtons("");
        }
    }//GEN-LAST:event_insertPolyPointsButtonActionPerformed

    private void createPolyByClickButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createPolyByClickButtonActionPerformed
        if(this.currentImage != null) {
            this.currentMode.clear();
            this.currentMode = new InsertPolyPointsOnClickAction(grid, pointManager, polylineManager);
            setToggleButtons(evt.getSource());
        } else {
            setToggleButtons("");
        }
    }//GEN-LAST:event_createPolyByClickButtonActionPerformed

    private void createPolylineButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createPolylineButtonActionPerformed
        if(this.currentImage != null) {
            String newName = (String) JOptionPane.showInputDialog("Write new polyline name (please pay attention, you cannot change this name after creation!)","Polyline" + (++idcounter));
            if(newName != null) {
                Polyline p = new DefaultPolyline(newName, grid);
                // Add to container
                this.polylines.add(p);
                // Deselect previous and select current
                if(this.currentPolyline != null)
                    this.currentPolyline.deselect();
                this.currentPolyline = p;
                p.select();
                // Attach to draw
                this.polylineManager.attach(p);
                updatePolyList();

                this.createPolyByClickButton.doClick();
            }
        } else {
            JOptionPane.showMessageDialog(mainPanel, "Open an image first!");
        }
    }//GEN-LAST:event_createPolylineButtonActionPerformed

    private void jList1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseClicked
        int selIndex = this.jList1.locationToIndex(evt.getPoint());
        if(selIndex != -1) {
            this.jList1.setSelectedIndex(selIndex);
            Polyline selPol = (Polyline) this.jList1.getModel().getElementAt(this.jList1.getSelectedIndex());
            // Deselect previous and select current
            if(this.currentPolyline != null)
                this.currentPolyline.deselect();
            this.currentPolyline = selPol;
            selPol.select();
        }
    }//GEN-LAST:event_jList1MouseClicked

    private void removeSelPolylineButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeSelPolylineButtonActionPerformed
        if(this.currentPolyline != null) {
            this.currentPolyline.deselect();
            this.polylines.remove(this.currentPolyline);
            this.polylineManager.detach(this.currentPolyline);
            DefaultListModel lm = new DefaultListModel();
            // Update List
            for(Polyline ply : this.polylines)
                lm.addElement(ply);
            this.jList1.setModel(lm);
            this.jList1.setSelectedIndex(this.polylines.size() - 1);
            this.jList1.invalidate();
            
            this.createPointsButton.doClick();
        }
    }//GEN-LAST:event_removeSelPolylineButtonActionPerformed

    private void gridBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_gridBoxActionPerformed
        if(this.currentImage != null) {
            this.grid.setGridSize(((Integer) this.gridBox.getSelectedItem()).intValue());
        }
    }//GEN-LAST:event_gridBoxActionPerformed

    private void scaleBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scaleBoxActionPerformed
        if(this.currentImage != null) {
            this.grid.setScale(((Integer) this.scaleBox.getSelectedItem()).intValue());
            this.imageContainerPanel.setPreferredSize(new Dimension(  (int)(currentImage.getWidth() * this.grid.getScale())  , (int) (currentImage.getHeight() * this.grid.getScale()) ) );
            this.imageContainerPanel.setSize(new Dimension(  (int)(currentImage.getWidth() * this.grid.getScale())  , (int) (currentImage.getHeight() * this.grid.getScale()) ));
            this.jScrollPane1.validate();
            this.imageContainerPanel.repaint();
        }
    }//GEN-LAST:event_scaleBoxActionPerformed

    private void snapToggleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_snapToggleButtonActionPerformed
        if(this.currentImage != null) {
            if(this.snapToggleButton.isSelected())
                this.grid.setSnap(true);
            else
                this.grid.setSnap(false);
        }
    }//GEN-LAST:event_snapToggleButtonActionPerformed

    private void loadProjectMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadProjectMenuActionPerformed
        if(currentImage != null) {
            this.loadProjectDialog.setMultiSelectionEnabled(false);
            int res = this.loadProjectDialog.showOpenDialog(this.mainPanel);
            File f = loadProjectDialog.getSelectedFile();
            if(res == JFileChooser.APPROVE_OPTION && f != null) {
                try {
                    
                    // Import data
                    Importer imp = new DefaultScaleSnapImporter();
                    imp.load(f.getPath());
                    // Clear grid
                    clearGridData();
                    // Create new grid informations
                    this.grid = (ScaleSnapGrid) imp.getGrid();
                    this.polylineManager = new DrawPolylineManager(new DefaultDrawPolylineFactory());
                    for(Polyline pp : imp.getPolylines()) {
                        polylines.add(pp);
                        polylineManager.attach(pp);
                    }
                    this.pointManager = new DrawPointManager(grid, new DefaultDrawPointFactory(1, 15));
                    // End with loading
                    endLoading();
                    updatePolyList();
                } catch (Exception e) {
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this.mainPanel,"Exception while loading file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
                }
            }
        } else {
            JOptionPane.showMessageDialog(this.mainPanel,"Cannot load before loading an image","Error...",JOptionPane.ERROR_MESSAGE);      
        }
}//GEN-LAST:event_loadProjectMenuActionPerformed

    private void saveProjectMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveProjectMenuActionPerformed
        if(grid != null) {
            this.saveDialog.setMultiSelectionEnabled(false);
            int res = this.saveDialog.showSaveDialog(this.mainPanel);
            File f = saveDialog.getSelectedFile();
            if(res == JFileChooser.APPROVE_OPTION && f != null) {
                try {
                    Exporter exp = new DefaultExporter(f.getPath());
                    exp.export(grid, polylines.toArray(new Polyline[this.polylines.size()]));
                } catch (Exception e) {
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this.mainPanel,"Exception while saving file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
                }
            }
        } else {
            JOptionPane.showMessageDialog(this.mainPanel,"Cannot save an empty project","Error...",JOptionPane.ERROR_MESSAGE);      
        }
}//GEN-LAST:event_saveProjectMenuActionPerformed

    private void exportProjectMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exportProjectMenuActionPerformed
        if(grid != null) {
            this.exportDialog.setMultiSelectionEnabled(false);
            this.exportDialog.setFileFilter(new FileNameExtensionFilter("PLaSM File","psm"));
            int res = this.exportDialog.showSaveDialog(this.mainPanel);
            File f = exportDialog.getSelectedFile();
            if(res == JFileChooser.APPROVE_OPTION && f != null) {
                try {
                    Exporter exp = new PlasmExporter(f.getPath());
                    exp.export(grid, polylines.toArray(new Polyline[this.polylines.size()]));
                } catch (Exception e) {
                    /* Warn the user about file */
                    JOptionPane.showMessageDialog(this.mainPanel,"Exception while exporting file " + f.getPath() + ": " + e.getMessage(),"Error...",JOptionPane.ERROR_MESSAGE);
                }
            }
        } else {
            JOptionPane.showMessageDialog(this.mainPanel,"Cannot export an empty project","Error...",JOptionPane.ERROR_MESSAGE);      
        }
}//GEN-LAST:event_exportProjectMenuActionPerformed

    private void clearGridData() {
        if(grid != null) {
            // Clear data
            for(Polyline p : polylines)
                this.polylineManager.detach(p);
            this.polylines.clear();
            this.currentPolyline = null;
            this.polylineManager = null;
            this.pointManager = null;
            this.grid = null;
            if(this.imageComponent != null) {
                this.imageContainerPanel.remove(this.imageComponent);
                this.imageComponent.validate();
                this.imageComponent.removeAll();
                this.imageComponent = null;
            }
            this.currentMode = new NoOpAction();
        }
    }
    
    private void endLoading() {
            // Scale
            Integer[] scales = new Integer[] {new Integer(1) , new Integer(2) , new Integer(4) };
            DefaultComboBoxModel scaleModel = new DefaultComboBoxModel(scales);
            this.scaleBox.setModel(scaleModel);
            this.scaleBox.setSelectedIndex(0);
            this.scaleBox.invalidate();
            
            // Grid
            Integer[] grids = new Integer[] {new Integer(1) , new Integer(2) , new Integer(4), new Integer(10) };
            DefaultComboBoxModel gridModel = new DefaultComboBoxModel(grids);
            this.gridBox.setModel(gridModel);
            this.gridBox.setSelectedIndex(0);
            this.gridBox.invalidate();
            
            imageComponent = new ImageDrawComponent(currentImage,pointManager,polylineManager,grid);
            
            imageComponent.addMouseListener(this);
            imageComponent.addMouseMotionListener(this);
            
            this.imageContainerPanel.add(BorderLayout.CENTER,imageComponent);
            this.imageContainerPanel.setPreferredSize(new Dimension(currentImage.getWidth(),currentImage.getHeight()));
            this.imageContainerPanel.setSize(new Dimension(currentImage.getWidth(),currentImage.getHeight()));
            this.jScrollPane1.validate();
            this.imageContainerPanel.repaint();
    }
    
    private void setToggleButtons(Object c) {
        this.noActionButton.setSelected(c == noActionButton);
        this.createPointsButton.setSelected(c == createPointsButton);
        this.createPolyByClickButton.setSelected(c == createPolyByClickButton);
        this.insertPolyPointsButton.setSelected(c == insertPolyPointsButton);
        if(this.currentMode != null)
            this.statusMessageLabel.setText(this.currentMode.getDescription());
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToggleButton createPointsButton;
    private javax.swing.JToggleButton createPolyByClickButton;
    private javax.swing.JButton createPolylineButton;
    private javax.swing.JMenuItem exportProjectMenu;
    private javax.swing.JComboBox gridBox;
    private javax.swing.JPanel imageContainerPanel;
    private javax.swing.JToggleButton insertPolyPointsButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JList jList1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JMenuItem loadImageMenuItem;
    private javax.swing.JMenuItem loadProjectMenu;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JToggleButton noActionButton;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JButton removeSelPolylineButton;
    private javax.swing.JMenuItem saveProjectMenu;
    private javax.swing.JComboBox scaleBox;
    private javax.swing.JToggleButton snapToggleButton;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JPanel toolPanel;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;

    public void mouseClicked(MouseEvent e) {
       
    }

    public void mousePressed(MouseEvent e) {
         if(e.getButton() == MouseEvent.BUTTON1)
            this.currentMode.leftMouseButtonClick(e, this.currentPolyline);
        else if (e.getButton() == MouseEvent.BUTTON2)
            this.currentMode.middleMouseButtonClick(e, currentPolyline);
        else if (e.getButton() == MouseEvent.BUTTON3)
            this.currentMode.rightMouseButtonClick(e, currentPolyline);
        else
            System.out.println("Mouse button not supported"); 
    }

    public void mouseReleased(MouseEvent e) {
        
    }

    public void mouseEntered(MouseEvent e) {
        
    }

    public void mouseExited(MouseEvent e) {
        
    }

    public void mouseDragged(MouseEvent e) {
        this.currentMode.dragMouse(e, currentPolyline);
    }

    public void mouseMoved(MouseEvent e) {
        
    }

    private void updatePolyList() {
        // Add to list component
                DefaultListModel lm = new DefaultListModel();
                for(Polyline ply : this.polylines)
                    lm.addElement(ply);
                this.jList1.setModel(lm);
                this.jList1.setSelectedIndex(this.polylines.size() - 1);
                this.jList1.invalidate();
    }
}
